"{ Package: 'stx:goodies/xtreams/core/tests' }"

"{ NameSpace: Xtreams }"

ReadingWritingTest subclass:#FiniteReadingWritingTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Core-Tests'
!

FiniteReadingWritingTests comment:'These are tests that require a stream that can be effectively closed, i.e. not pointer or shared queue stream.

'
!


!FiniteReadingWritingTests methodsFor:'tests - core'!

testGet
	self output write: #[ 1 2 3 4 ]; close.
	self assert: self input get = 1.
	self assert: self input get = 2.
	self assert: self input get = 3.
	self assert: self input get = 4.
	self should: [self input get] raise: Incomplete
!

testGetPastEnd
	self output put: 120.
	self output close.
	self assert: self input get = 120.
	self should: [self input get] raise: Incomplete
!

testRead

	self output write: #[ 1 2 3 4 ]; close.
	self assert: (self input read: 3) = #[1 2 3].
	self assert: ([self input read: 2. false] on: Incomplete do: [:exception |
		exception collection first = 4.
		exception collection size = 2.
		exception count = 1.
		true])
!

testReadIntoAt
	| result |
	self output write: #[ 1 2 3 4 ]; close.
	result := ByteArray new: 8 withAll: 255.
	self input read: 3 into: result at: 3.
	self assert: result = #[255 255 1 2 3 255 255 255].
	self input read: 1 into: result at: 6.
	self assert: result = #[255 255 1 2 3 4 255 255].
	self should: [self input get] raise: Incomplete.
	self should: [self input read: 3 into: result at: 5] raise: Incomplete.
	self assert: result = #[255 255 1 2 3 4 255 255].
!

testReadIntoAtPastEnd
	| result |

	self output write: #[ 1 2 3 4 ]; close.
	self input read: 1.
	result := ByteArray new: 4 withAll: 255.
	self assert: ([self input read: 4 into: result at: 1. false] on: Incomplete do: [ :eos | eos count = 3 ]).
	self assert: result = #[2 3 4 255]
!

testRest

	self output write: #[ 1 2 3 4 ]; close.
	self assert: self input rest = #[1 2 3 4].
	self assert: self input rest = #[]
!

testWriteStream
	| data |
	data := #[ 1 2 3 4 ] reading.
	self assert: (self output write: data) = 4.
	self shouldnt: [ self output write: data ] raise: Incomplete.
	self output close.
	self assert: (self input read: 4) = #[ 1 2 3 4 ].
	self should: [self input get] raise: Incomplete
!

testWriteTruncating

	self output positioning
		write: #[1 2 3 4 5 6];
		-- 3;
		write: #[1 2];
		close.
	self assert: self input rest = #[1 2 3 1 2].
! !

!FiniteReadingWritingTests methodsFor:'tests - core - positioning'!

testReadPositioningSeek

	| record in |
	record := #[ 1 2 3 4 5 6 7 8 9 0 ].
	self output write: record; close.
	in := self input positioning.
	self assert: (in ++ 7) = 7.
	self assert: in position = 7.
	self assert: (in -- 3) = 3.
	self assert: in position = 4.
	self assert: ([ in ++ record size ] on: Incomplete do: [:ex | ex count]) = (record size - 4).
!

testReadPositioningSeekFrom

	| record in |
	record := #[ 1 2 3 4 5 6 7 8 9 0 ].
	self output write: record; close.
	in := self input positioning.
	self assert: (in += 3) = 3.
	self assert: in position = 3.
	self assert: (in -= 4) = 4.
	self assert: in position = 6.
	self assert: ([ in -= (record size + 6) ] on: Incomplete do: [:ex | ex count]) = record size.
	self assert: in position = 0.
	self assert: ([ in += (record size + 6) ] on: Incomplete do: [:ex | ex count]) = record size.
	self assert: in position = record size.
!

testWritePositioningSeek

	| record out |
	record := #[ 1 2 3 4 5 6 7 8 9 0 ].
	out := self output positioning.
	out     write: record.
	self assert: (out -- 7) = 7.
	self assert: out position = 3.
	self assert: (out ++ 3) = 3.
	self assert: out position = 6.
	self assert: ([ out ++ record size ] on: Incomplete do: [:ex | ex count]) = (record size - 6).
	out close.
	self assert: (self input read: record size) = record
!

testWritePositioningSeekFrom

	| record out |
	record := #[ 1 2 3 4 5 6 7 8 9 0 ].
	out := self output positioning.
	out     write: record.
	self assert: (out += 3) = 3.
	self assert: out position = 3.
	self assert: (out -= 4) = 4.
	self assert: out position = 6.
	self assert: ([ out -= (record size + 6) ] on: Incomplete do: [:ex | ex count]) = record size.
	self assert: out position = 0.
	self assert: ([ out += (record size + 6) ] on: Incomplete do: [:ex | ex count]) = record size.
	self assert: out position = record size.
	out close.
	self assert: (self input read: record size) = record
! !

!FiniteReadingWritingTests methodsFor:'tests - enumeration'!

testCollect

	| data result |
	data := ByteArray withAll: (0 to: 15).
	self output write: data; close.
	result := self input collect: [ :e | e * e ].
	self assert: result size = data size.
	data with: result do: [ :in :out | self assert: in * in = out ]
!

testDetect

	| data result |
	data := ByteArray withAll: (1 to: 15).
	self output write: data; close.
	result := self input detect: [ :e | e \\ 4 = 0 ].
	self assert: result = 4.
	result := self input detect: [ :e | e \\ 4 = 0 ].
	self assert: result = 8.
	result := self input detect: [ :e | e \\ 4 = 0 ].
	self assert: result = 12.
	result := self input detect: [ :e | e \\ 4 = 0 ] ifNone: [ nil ].
	self assert: result = nil
!

testDo

	| data result i |
	data := ByteArray withAll: (0 to: 15).
	self output write: data; close.
	result := ByteArray new: data size.
	i := 1.
	self input do: [ :e | result at: i put: e * e. i := i + 1 ].
	data with: result do: [ :in :out | self assert: in * in = out ]
!

testSelect

	| data result |
	data := ByteArray withAll: (0 to: 15).
	self output write: data; close.
	result := self input select: [ :e | e odd ].
	self assert: result = (data select: [ :e | e odd ]).
! !


!FiniteReadingWritingTests methodsFor:'tests - seeking'!

testExploreInsideExplore
	| in |
	self output write: #[ 1 2 3 4 ]; close.
	in := self input positioning.
	self assert: (in explore: [(in explore: [in rest = #[1 2 3 4]]) & (in rest = #[1 2 3 4])]).
	self assert: in rest = #[1 2 3 4].
	self should: [in get] raise: Incomplete
!

testExploreRead
	| in |
	self output write: #[ 1 2 3 4 ]; close.
	in := self input positioning.
	self assert: (in explore: [in rest = #[1 2 3 4]]).
	self assert: in rest = #[1 2 3 4].
	self should: [in get] raise: Incomplete
!

testExploreSeekBackward
	| in |
	self output write: #[ 1 2 3 4 ]; close.
	in := self input positioning.
	self assert: (in explore: [in read: 3; -- 1. in rest = #[3 4]]).
	self assert: in rest = #[1 2 3 4].
	self should: [in get] raise: Incomplete
!

testExploreSeekBackwardPastBeginning
	| in |
	self output write: #[ 1 2 3 4 ]; close.
	in := self input positioning.
	self should: [ in explore: [in read: 2; -- 4 ] ] raise: Incomplete.
	self should: [ in explore: [in read: 2; -= 6 ] ] raise: Incomplete.
	self assert: in rest = #[1 2 3 4].
	self should: [in get] raise: Incomplete
!

testExploreSeekForward
	| in |
	self output write: #[ 1 2 3 4 ]; close.
	in := self input positioning.
	self assert: (in explore: [in ++ 2. in rest = #[3 4]]).
	self assert: in rest = #[1 2 3 4].
	self should: [in get] raise: Incomplete
!

testExploreSeekForwardPastEnd
	| in |
	self output write: #[ 1 2 3 4 ]; close.
	in := self input positioning.
	self should: [ in explore: [in ++ 8] ] raise: Incomplete.
	self should: [ in explore: [in += 8] ] raise: Incomplete.
	self assert: in rest = #[1 2 3 4].
	self should: [in get] raise: Incomplete
!

testSeekBackward

	self output write: #[ 1 2 3 4 ]; close.
	self input read: 2.
	[self input -- 2] on: Error do: [:error |
		self deny: self input isPositionable].
	self input isPositionable
		ifTrue: [self assert: self input rest = #[1 2 3 4]]
		ifFalse:        [self assert: self input rest = #[3 4]].
!

testSeekBackwardPastBeginning

	self output write: #[ 1 2 3 4 ]; close.
	[[self input -- 1]
		on: Incomplete do: [:incomplete | ^self assert: incomplete count = 0]]
		on: Error do: [:error | ^self deny: self input isPositionable].

	"If neither exception was raised, that is a problem."
	self deny: true
!

testSeekForward

	self output write: #[ 1 2 3 4 ]; close.
	self assert: (self input ++ 2) = 2.
	self assert: self input rest = #[3 4].
!

testSeekForwardPastEnd

	self output write: #[ 1 2 3 4 ]; close.
	self input ++ 2.
	self assert: (
		[       self input ++ 4. false
		] on: Incomplete do: [ :eos |
			eos count = 2 ]).
	self should: [ self input get ] raise: Incomplete.
	self assert: (
		[       self input ++ 4. false
		] on: Incomplete do: [ :eos |
			eos count = 0 ]).
!

testWritePositioningInsert

	| message result |
	message := ByteArray withAll: (1 to: 20).
	(PositionWriteStream on: self output)
		write: message; write: message;
		+= message size;
		insert: message;
		-= 0;
		close.
	result := self input rest.
	self assert: result = (message, message, message)
!

testWritePositioningWithRingBuffer

	| message result |
	message := ByteArray withAll: (1 to: 20).
	(PositionWriteStream on: self output)
		buffer: (RingBuffer on: (ByteArray new: 5));
		write: message;
		close.
	result := self input rest.
	self assert: result = message
! !




!FiniteReadingWritingTests class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

